package cz.fidentis.analyst.distance;

import cz.fidentis.analyst.canvas.Canvas;
import cz.fidentis.analyst.visitors.mesh.HausdorffDistance;
import cz.fidentis.analyst.visitors.mesh.HausdorffDistance.Strategy;
import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.JComboBox;
import javax.swing.JTabbedPane;
import javax.swing.JToggleButton;

/**
 * Action listener for the curvature computation.
 * 
 * @author Radek Oslejsek
 */
public class DistanceAction extends AbstractAction {
    
    public static final String ACTION_COMMAND_SHOW_HIDE_PANEL = "show-hide distance control panel";
    public static final String ACTION_COMMAND_SHOW_HIDE_HEATMAP = "show-hide heatmap";
    public static final String ACTION_COMMAND_SET_DISTANCE_STRATEGY = "set strategy";
    public static final String ACTION_COMMAND_RELATIVE_ABSOLUTE_DIST = "switch abosulte-relative distance";
    
    private final DistancePanel controlPanel;
    private final JTabbedPane topControlPanel;
    private final Canvas canvas;
    
    private HausdorffDistance distVisitor = null;
    private String strategy = DistancePanel.STRATEGY_POINT_TO_POINT;
    private boolean relativeDist = false;
    
    /**
     * Constructor.
     * 
     * @param topControlPanel Top component for placing control panels
     * @param canvas OpenGL canvas
     */
    public DistanceAction(JTabbedPane topControlPanel, Canvas canvas) {
        this.canvas = canvas;
        this.controlPanel = new DistancePanel(this);
        this.topControlPanel = topControlPanel;
    }

    @Override
    public void actionPerformed(ActionEvent ae) {
        String action = ae.getActionCommand();
        
        switch (action) {
            case ACTION_COMMAND_SHOW_HIDE_PANEL:
                if (((JToggleButton) ae.getSource()).isSelected()) {
                    topControlPanel.addTab(DistancePanel.NAME, DistancePanel.ICON, controlPanel);
                    topControlPanel.setSelectedComponent(controlPanel); // focus
                } else {
                    topControlPanel.remove(controlPanel);
                }
                break;
            case ACTION_COMMAND_SHOW_HIDE_HEATMAP:
                if (((JToggleButton) ae.getSource()).isSelected()) {
                    setHeatmap();
                    canvas.getScene().getDrawableFace(1).setRenderHeatmap(true);
                } else {
                    canvas.getScene().getDrawableFace(1).setRenderHeatmap(false);
                }
                break;
            case ACTION_COMMAND_SET_DISTANCE_STRATEGY:
                strategy = (String) ((JComboBox) ae.getSource()).getSelectedItem();
                this.distVisitor = null; // recompute
                setHeatmap();
                break;
            case ACTION_COMMAND_RELATIVE_ABSOLUTE_DIST:
                this.relativeDist = ((JToggleButton) ae.getSource()).isSelected();
                this.distVisitor = null; // recompute
                setHeatmap();
                break;
            default:
                throw new UnsupportedOperationException(action);
        }
        canvas.renderScene();
    }
    
    protected void setHeatmap() {
        Strategy useStrategy;
        switch (strategy) {
            case DistancePanel.STRATEGY_POINT_TO_POINT:
                useStrategy = Strategy.POINT_TO_POINT;
                break;
            case DistancePanel.STRATEGY_POINT_TO_TRIANGLE:
                useStrategy = Strategy.POINT_TO_TRIANGLE_APPROXIMATE;
                break;
            default:
                throw new UnsupportedOperationException(strategy);
        }
        
        if (distVisitor == null) { 
            this.distVisitor = new HausdorffDistance(canvas.getScene().getDrawableFace(0).getModel(), useStrategy, relativeDist, true);
            canvas.getScene().getDrawableFace(1).getModel().compute(distVisitor);
        }
        
        canvas.getScene().getDrawableFace(1).setHeatMap(distVisitor.getDistances());
    }
}
